/**
  @file 
  @brief 
  BRT_AN_031 FT90x UART to USB BOMs

  Control BOMs device using FatFS
 */
/*
 * ============================================================================
 * History
 * =======
 * DP  Nov 2017 : Created
 * DP  May 2018 : Code tidy for release
 *
 * Copyright (C) Bridgetek Pte Ltd
 * ============================================================================
 *
 * This source code ("the Software") is provided by Bridgetek Pte Ltd 
 * ("Bridgetek") subject to the licence terms set out
 * http://brtchip.com/BRTSourceCodeLicenseAgreement/ ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of Bridgetek liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on Bridgetek's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, Bridgetek
 * has no liability in relation to those amendments.
 * ============================================================================
 */

/* INCLUDES ************************************************************************/

#include <stdint.h>
#include <string.h>

/* FT90x includes */
#include <ft900.h>
#include <ft900_usb.h>
#include <ft900_usbh_boms.h>
#include <ft900_startup_dfu.h>

/* UART support for printf output. */
#include "tinyprintf.h"

/* Support for FATFS */
#include "ff.h"
#include "diskio.h"

/* Support for RTC */
#include <time.h>

/* Program Header */
#include "main.h"

#include "uartrb.h"

/* Global Variables **************************************************************/

/* Fatfs resources */
FATFS fs;
FRESULT res;
FIL f;
FILINFO fno;
UINT towrite, written;
UINT read;
char* buffer[128];
DWORD fre_clust, fre_sect, tot_sect;
char label[24];

/* Context structure and handle for BOMS device. */
USBH_BOMS_context bomsCtx;
USBH_interface_handle hOpenDisk = 0;

/* Variables used in main program */
volatile int command_char_count, filename_char_count = 0;
uint16_t filedata_char_count;

char command[4];
char filename[256];
char filedata[1025];
char* data;

TCHAR getcwd[256];

int mount_init = 0;
int command_val = 0;
uint8_t test_res = 0;
uint16_t continuous_char_count = 0;
uint64_t count = 0;

uint32_t HIPID_value = 0;

/* RTC resources */
#define RTC_ENABLE
//#undef RTC_ENABLE

int rtc_year;
int rtc_month;
int rtc_mday;
int rtc_hour;
int rtc_min;
int rtc_sec;

/* enums **************************************************************************/
volatile enum {
	COMMAND = 0,
	FILENAME,
	FILEDATA,
	CONT_FILEDATA
} input_phase;

volatile enum boolean {
	FALSE = 0,
	TRUE
} command_entered;

/* MACROS **************************************************************************/

#define FDATE_DAY(fdate) ((fdate)&(0x1F))
#define FDATE_MONTH(fdate) (((fdate)>>5)&(0x0F))
#define FDATE_YEAR(fdate) ( (((fdate)>>9)&(0x7F)) + 1980 )

#define FTIME_HOUR(ftime) (((ftime)>>11)&(0x1F))
#define FTIME_MIN(ftime) (((ftime)>>5)&(0x3F))

/* FUNCTIONS ************************************************************************/

/** @name tfp_putc
 *  @details Machine dependent putc function for tfp_printf (tinyprintf) library.
 *  @param p Parameters (machine dependent)
 *  @param c The character to write
 */
void tfp_putc(void* p, char c)
{
	uart_write((ft900_uart_regs_t*)p, (uint8_t)c);
}

void ISR_timer(void)
{
	struct tm time = { 0 };
	uint32_t total_seconds_elapsed = 0;
	uint8_t seconds_elapsed = 0;
	uint8_t minutes_elapsed = 0;
	uint8_t hours_elapsed = 0;

	if (timer_is_interrupted(timer_select_a))
	{
		// Call USB timer handler to action transaction and hub timers.
		USBH_timer();
	}

	if (timer_is_interrupted(timer_select_b))
	{

		if (HIPID_value == 0x9000001) //RevB
		{


			/* Read the time... */
			rtc_read(&time);

			total_seconds_elapsed = time.tm_sec >> 5; /* Cut off the fractional seconds */
			seconds_elapsed = total_seconds_elapsed % 60;
			minutes_elapsed = (total_seconds_elapsed / 60) % 60;
			hours_elapsed = (total_seconds_elapsed / (60*60));

			if (hours_elapsed >= 24)
			{
				rtc_mday++;
				hours_elapsed = hours_elapsed - 24;
				time.tm_sec = ((((hours_elapsed*60*60))+(minutes_elapsed*60)+seconds_elapsed)<<5);
				rtc_write(&time);

				//Months with 31 days
				if (((rtc_month == 1)||(rtc_month == 3)||(rtc_month == 5)||(rtc_month == 8)||(rtc_month == 10)||(rtc_month == 12))&&(rtc_mday >= 31))
				{
					if(rtc_month == 12)
					{
						rtc_month = 1;
						rtc_year++;
					}
					else
					{
						rtc_month++;
					}
					rtc_mday = 1;
				}
				//Months with 30 days
				if (((rtc_month == 4)||(rtc_month == 6)||(rtc_month == 9)||(rtc_month == 11))&&(rtc_mday >= 30))
				{
					rtc_month++;
					rtc_mday = 1;
				}
				//February: 28 or 29 days
				if ((rtc_month == 2)&&(rtc_mday >= 28))
				{
					if(rtc_mday == 29)
					{
						if((rtc_year+1980)%4 == 0)
						{
							if( (rtc_year+1980)%100 == 0)
							{
								if ((rtc_year+1980)%400 == 0)
								{
									//leap year, do nothing, mday is already incremented
								}
							}
						}
						else
						{
							//no leap year
							rtc_month++;
							rtc_mday = 1;
						}
					}
					else
					{
						rtc_month++;
						rtc_mday = 1;
					}
				}
			}
		}
	}

}


/** List the directory 
 *  @param path The directory to list
 *  @return The status of the operation */
FRESULT dir(char* path)
{
	FRESULT res;
	FILINFO info;
	DIR dir;
	char * fname;

	uint8_t day, month, hour, min;
	uint16_t year;
	uint32_t totalsize = 0, totalfiles = 0;

	res = f_opendir(&dir, path);
	if (FR_OK == res)
	{
		/* Display a header */
		tfp_printf("\r\n" "path = %s",path);
		tfp_printf("\r\n");
		tfp_printf("DD/MM/YYYY HH:MM               Size Filename\r\n");

		for(;;)
		{
			res = f_readdir(&dir, &info);

			/* Break if we have finished or an error has occurred */
			if ((FR_OK != res) || (info.fname[0] == 0))
				break;

			/* Assign fname with the proper filename */
			fname = info.fname;

			/* Work out the date and time the file was last written to */
			day   = FDATE_DAY(info.fdate);
			month = FDATE_MONTH(info.fdate);
			year  = FDATE_YEAR(info.fdate);

			hour  = FTIME_HOUR(info.ftime);
			min   = FTIME_MIN(info.ftime);

			/* Count the number of files in this folder */
			if (!(info.fattrib & AM_DIR))
				totalfiles++;

			/* Count the total size of all files in this folder */
			totalsize += info.fsize;

			/* Display the item */
			tfp_printf("%02d/%02d/%04d %02d:%02d   %s %10ld ",
					day, month, year, hour, min,
					(info.fattrib & AM_DIR) ? "<DIR>" : "     ",
							info.fsize);
			tfp_printf("%s \r\n", fname);
		}

		if (FR_OK == res)
		{
			/* Display the summary if things were ok */
			tfp_printf("  %10ld File(s)     %10ld bytes\r\n",
					totalfiles, totalsize);
		}

		f_closedir(&dir);
	}

	return res;
}


/* FatFS Functions ******************/

/** Initialise a drive
 *  @param pdrv Physical Drive number
 *  @return Disk Status */
DSTATUS disk_initialize(BYTE pdrv)
{
	DSTATUS stat = 0;
	int8_t status;

	status = USBH_BOMS_init(hOpenDisk, 0, &bomsCtx);

	if (status != USBH_BOMS_OK)
	{
		tfp_printf("ERROR BOMS_init \r\n");
#ifdef DEBUG
		tfp_printf("BOMS device could not be initialised: %d\r\n", status);
#endif
		stat = STA_NOINIT;
		hOpenDisk = 0;
	}

	return stat;
}

/** Disk Status
 *  @param pdrv Physical Drive number
 *  @return Disk Status */
DSTATUS disk_status(BYTE pdrv)
{
	DSTATUS stat = 0;

	if (0 == hOpenDisk)
	{
		stat |= STA_NOINIT;
	}

	if (USBH_BOMS_status(&bomsCtx) != USBH_BOMS_OK)
	{
		stat |= STA_NODISK;
	}

	return stat;
}

/** Read sector(s) from disk
 *  @param pdrv Physical Drive number
 *  @param buff Data buffer to store into
 *  @param sector The logical sector address
 *  @param count The number of sectors to read
 *  @return Disk Status */
DRESULT disk_read (BYTE pdrv, BYTE* buff, DWORD sector, UINT count)
{
	DRESULT res = RES_OK;

	if (USBH_BOMS_read(&bomsCtx, sector, USBH_BOMS_BLOCK_SIZE * count, buff) != USBH_BOMS_OK)
	{
		res = RES_ERROR;
	}

	return res;
}

#if _USE_WRITE
/** Write sector(s) to the disk
 *  @param pdrv Physical Drive number
 *  @param buff Data buffer to write to the disk
 *  @param sector The logical sector address
 *  @param count The number of sectors to write
 *  @return Disk Status */
DRESULT disk_write (BYTE pdrv, const BYTE* buff, DWORD sector, UINT count)
{
	DRESULT res = RES_OK;

	if (USBH_BOMS_write(&bomsCtx, sector, USBH_BOMS_BLOCK_SIZE * count, (uint8_t *)buff) != USBH_BOMS_OK)
	{
		res = RES_ERROR;
	}

	return res;
}
#endif

#if _USE_IOCTL
/** Disk IO Control
 *  @param pdrv Physical Drive Number
 *  @param cmd Control Code
 *  @param buff Buffer to send/receive control data 
 *  @return Disk Status */
DRESULT disk_ioctl (BYTE pdrv, BYTE cmd, void* buff)
{
	DRESULT res = RES_OK;

	/* Not Supported */

	return res;
}
#endif

#if _FS_READONLY == 0
/** Get the current time
 *  @return The time in the following format:
 *          bit[31:25] = Year from 1980 (0..127),
 *          bit[24:21] = Month (1..12),
 *          bit[20:16] = Day of the Month (1..31),
 *          bit[15:11] = Hour (0..23),
 *          bit[10:5]  = Minute (0..59),
 *          bit[4..0]  = Second / 2 (0..29) */
DWORD get_fattime(void)
{
	if (HIPID_value == 0x9000001) //RevB
	{
		struct tm time = { 0 };
		int8_t rtc_res = 0;

		uint32_t total_seconds_elapsed = 0;
		uint8_t seconds_elapsed = 0;
		uint8_t minutes_elapsed = 0;
		uint8_t hours_elapsed = 0;

		/* Read the time... */
		rtc_res = rtc_read(&time);

		if (rtc_res != 0)
		{
			return 0;
		}

		total_seconds_elapsed = time.tm_sec >> 5; /* Cut off the fractional seconds */
		seconds_elapsed = total_seconds_elapsed % 60;
		minutes_elapsed = (total_seconds_elapsed / 60) % 60;
		hours_elapsed = (total_seconds_elapsed / (60*60));

		return ((rtc_year<<25)|(rtc_month<<21)|(rtc_mday<<16)|(hours_elapsed<<11)|(minutes_elapsed<<5)|seconds_elapsed);
	}
	else if (HIPID_value == 0x9000002) //RevC
	{
		struct tm time = { 0 };
		uint32_t return_time = 0;
		int8_t rtc_res = 0;

		/* Read the time... */
		rtc_res = rtc_read(&time);

		if (rtc_res != 0)
		{
			return 0;
		}

		return_time = ((time.tm_year+20)<<25)|(time.tm_mon<<21)|(time.tm_mday<<16)|(time.tm_hour<<11)|(time.tm_min<<5)|time.tm_sec;

		return return_time;

	}

}
#endif

void reset_vars(void){

	/* Reset all variables */
	int i = 0;
	input_phase = COMMAND;
	command_entered = FALSE;
	command_char_count = 0;
	filename_char_count = 0;
	filedata_char_count = 0;
	command_val = 0;
	test_res = 0;
	continuous_char_count = 0;
	count = 0;

	for (i=0;i<4;i++){
		command[i] = '\0';
	}
	for (i=0;i<14;i++){
		filename[i] = '\0';
	}
	for (i=0;i<1024;i++){
		filedata[i] = '\0';
	}

	uartrb_flush_read(UART0);

}

int8_t fs_testing(USBH_interface_handle hBOMS)
{
	int8_t cmd_res;

	if (mount_init == 0){
		/* Initialise FatFS */
		hOpenDisk = hBOMS;
		res = f_mount(&fs, "", 0);
		if (FR_OK != res)
		{
			tfp_printf("ERROR Mount %d\r\n", res);
#ifdef DEBUG
			tfp_printf("Unable to mount File System\r\n");
#endif
			return 1;
		}
		mount_init = 1;
#ifdef DEBUG
		tfp_printf("Mount File System OK\r\n");
#endif
	}

	if(command_entered == TRUE)
	{

		if (command[0]=='d' && command[1]=='i' && command[2]=='r'){ command_val = 1;}		//dir
		else if (command[0]=='m' && command[1]=='k' && command[2]=='d'){ command_val = 2;}	//mkd
		else if (command[0]=='c' && command[1]=='d'){ command_val = 3;}						//cd
		else if (command[0]=='g' && command[1]=='e' && command[2]=='t'){ command_val = 4;}	//get
		else if (command[0]=='c' && command[1]=='r' && command[2]=='f'){ command_val = 5;}	//crf
		else if (command[0]=='d' && command[1]=='e' && command[2]=='l'){ command_val = 6;}	//del
		else if (command[0]=='w' && command[1]=='t' && command[2]=='f'){ command_val = 7;}	//wtf
		else if (command[0]=='r' && command[1]=='d' && command[2]=='f'){ command_val = 8;}	//rdf
		else if (command[0]=='u' && command[1]=='n' && command[2]=='m'){ command_val = 9;}	//unm
		else if (command[0]=='?'){ command_val = 10;}										//?
		else if (command[0]=='s' && command[1]=='i' && command[2]=='z'){ command_val = 11;}	//siz
		else if (command[0]=='r' && command[1]=='n' && command[2]=='m'){ command_val = 12;}	//rnm
		else if (command[0]=='f' && command[1]=='r' && command[2]=='e'){ command_val = 13;}	//fre
		else if (command[0]=='g' && command[1]=='e' && command[2]=='l'){ command_val = 14;}	//gel
		else if (command[0]=='s' && command[1]=='e' && command[2]=='l'){ command_val = 15;}	//sel
		else if (command[0]=='t' && command[1]=='i' && command[2]=='m'){ command_val = 16;}	//tim
		else if (command[0]=='r' && command[1]=='t' && command[2]=='c'){ command_val = 17;}	//rtc
		else if (command[0]=='d' && command[1]=='f' && command[2]=='u'){ command_val = 18;}	//dfu
		else if (command[0]=='v' && command[1]=='e' && command[2]=='r'){ command_val = 19;}	//ver

		switch(command_val) {
		case 1:
		{
			/* Command dir */
			cmd_res = cmd_dir();
			if (cmd_res != 0)
			{
				return 1;
			}
		}
		break;

		case 2:
		{
			/* Command mkd */
			cmd_res = cmd_mkd();
			if (cmd_res != 0)
			{
				return 1;
			}
		}
		break;
		case 3:
		{
			/* Command cd */
			cmd_res = cmd_cd();
			if (cmd_res != 0)
			{
				return 1;
			}
		}
		break;
		case 4:
		{
			/* Command get */
			cmd_res = cmd_get();
			if (cmd_res != 0)
			{
				return 1;
			}
		}
		break;
		case 5:
		{
			/* Command crf */
			cmd_res = cmd_crf();
			if (cmd_res != 0)
			{
				return 1;
			}
		}
		break;
		case 6:
		{
			/* Command del */
			cmd_res = cmd_del();
			if (cmd_res != 0)
			{
				return 1;
			}
		}
		break;
		case 7:
		{
			/* Command wtf */
			cmd_res = cmd_wtf();
			if (cmd_res != 0)
			{
				return 1;
			}
		}
		break;
		case 8:
		{
			/* Command rdf */
			cmd_res = cmd_rdf();
			if (cmd_res != 0)
			{
				return 1;
			}
		}
		break;
		case 9:
		{
			/* Command unm */
			cmd_res = cmd_unm();
			if (cmd_res == 2)
			{
				return 2;
			}
		}
		break;
		case 10:
		{
			/* Command ? (help) */
			cmd_help();
		}
		break;
		case 11:
		{
			/* Command siz */
			cmd_res = cmd_siz();
			if (cmd_res != 0)
			{
				return 1;
			}
		}
		break;
		case 12:
		{
			/* Command rnm */
			cmd_res = cmd_rnm();
			if (cmd_res != 0)
			{
				return 1;
			}
		}
		break;
		case 13:
		{
			/* Command fre */
			cmd_res = cmd_fre();
			if (cmd_res != 0)
			{
				return 1;
			}
		}
		break;
		case 14:
		{
			/* Command gel */
			cmd_res = cmd_gel();
			if (cmd_res != 0)
			{
				return 1;
			}
		}
		break;
		case 15:
		{
			/* Command sel */
			cmd_res = cmd_sel();
			if (cmd_res != 0)
			{
				return 1;
			}
		}
		break;
		case 16:
		{
			/* Command tim */
			cmd_res = cmd_tim();
			if (cmd_res != 0)
			{
				return 1;
			}
		}
		break;
		case 17:
		{
			/* Command rtc */
			cmd_res = cmd_rtc();
			if (cmd_res != 0)
			{
				return 1;
			}
		}
		break;
		case 18:
		{
			/* Command dfu */
			cmd_res = cmd_dfu();
			if (cmd_res != 0)
			{
				return 1;
			}
		}
		break;
		case 19:
		{
			/* Command ver */
			cmd_res = cmd_ver();
			if (cmd_res != 0)
			{
				return 1;
			}
		}
		break;
		default:
		{
			/* Invalid Command */
			tfp_printf( "\r\n");
#ifdef DEBUG
			tfp_printf( "\r\n" "Invalid Command \r\n\r\n");
#endif
			delayms(100);
			return 1;
		}
		break;
		}

	}
	return 0;
}

int8_t cmd_dir(void){
	/* Command dir */
	tfp_printf("\r\n");

	/* List a directory */
	res = dir(filename);

	if (res != FR_OK){
		tfp_printf("ERROR DIR %d\r\n", res);
#ifdef DEBUG
		tfp_printf("DIR Error\r\n");
		print_error_code(res);
#endif
		return 1;
	}

	reset_vars();
	tfp_printf("> \r\n");
	delayms(100);

	return 0;
}

int8_t cmd_mkd(void){
	/* Command mkd */
	tfp_printf("\r\n");

	/* Make a new directory */
	res = f_mkdir(filename);
	if (FR_OK == res)
	{
#ifdef DEBUG
		tfp_printf("Directory created OK \r\n");
#endif
	}
	else
	{
		tfp_printf("ERROR MKD %d\r\n", res);
#ifdef DEBUG
		tfp_printf("Error creating directory \r\n");
		print_error_code(res);
#endif
		return 1;
	}

	reset_vars();
	tfp_printf("> \r\n");
	delayms(100);

	return 0;
}

int8_t cmd_cd(void){
	/* Command cd */
	tfp_printf("\r\n");

	/* Change the current directory */
	res = f_chdir(filename);
	if (FR_OK == res)
	{
#ifdef DEBUG
		tfp_printf("Directory changed OK \r\n");
#endif
	}
	else
	{
		tfp_printf("ERROR CD %d\r\n", res);
#ifdef DEBUG
		tfp_printf("Error changing directory \r\n");
		print_error_code(res);
#endif
		return 1;
	}

	reset_vars();
	tfp_printf("> \r\n");
	delayms(100);

	return 0;
}

int8_t cmd_get(void){
	/* Command get */
	tfp_printf("\r\n");

	/* Retrieve the current directory  */
	if (fs.fs_type == FS_EXFAT)
	{
		tfp_printf("ERROR GET 1 \r\n");
#ifdef DEBUG
		tfp_printf("Get current directory currently not supported on ExFAT \r\n");
#endif
		return 1;
	}
	else
	{
		res = f_getcwd(getcwd, sizeof getcwd / sizeof *getcwd);
		if (FR_OK == res)
		{
#ifdef DEBUG
			tfp_printf("Retrieving the current directory OK \r\n");
#endif
			tfp_printf("cwd: %s\r\n", getcwd);
		}
		else
		{
			tfp_printf("ERROR GET %d\r\n", res);
#ifdef DEBUG
			tfp_printf("Error getting current directory \r\n");
			print_error_code(res);
#endif
			return 1;
		}
	}

	reset_vars();
	tfp_printf("> \r\n");
	delayms(100);

	return 0;
}

int8_t cmd_crf(void){
	/* Command crf */
	tfp_printf("\r\n");

	/* Check to see if the file is there */
	res = f_stat(filename, NULL);
	if (FR_OK == res)
	{
		tfp_printf("ERROR CRF 1 %d \r\n", res);
#ifdef DEBUG
		tfp_printf("File already exists. \r\n");
#endif
		return 1;
	}

	/* Create new file */
#ifdef DEBUG
	tfp_printf("Creating new file \r\n");
#endif
	res = f_open(&f, filename, FA_CREATE_NEW);
	if (FR_OK != res)
	{
		tfp_printf("ERROR CRF 2 %d \r\n", res);
#ifdef DEBUG
		tfp_printf("Problem creating file \r\n");
		print_error_code(res);
#endif
		return 1;
	}

	/* Close the file */
#ifdef DEBUG
	tfp_printf("Closing \r\n");
#endif
	res = f_close(&f);
	if (FR_OK != res)
	{
		tfp_printf("ERROR CRF 3 %d \r\n", res);
#ifdef DEBUG
		tfp_printf("Error closing \r\n");
		print_error_code(res);
#endif
		return 1;
	}

	reset_vars();
	tfp_printf("> \r\n");
	delayms(100);

	return 0;
}

int8_t cmd_del(void){
	/* Command del */
	tfp_printf("\r\n");

	/* Check to see if the file/directory is there */
	res = f_stat(filename, NULL);
	if (FR_OK == res)
	{
#ifdef DEBUG
		tfp_printf("File or directory exists. \r\n");
#endif
	}
	else
	{
		tfp_printf("ERROR DEL 1 %d\r\n", res);
#ifdef DEBUG
		tfp_printf("Error file/directory does no exist \r\n");
		print_error_code(res);
#endif
		return 1;
	}

	/* Delete the file/directory */
	res = f_unlink(filename);

	if (FR_OK == res)
	{
#ifdef DEBUG
		tfp_printf("Delete successful. \r\n");
#endif
	}
	else
	{
		tfp_printf("ERROR DEL 2 %d \r\n", res);
#ifdef DEBUG
		tfp_printf("Delete Unsuccessful. \r\n");
		print_error_code(res);
#endif
		return 1;
	}

	reset_vars();
	tfp_printf("\r\n" "> \r\n");
	delayms(100);

	return 0;
}

int8_t cmd_wtf(void){
	/* Command wtf */
	tfp_printf("\r\n");

	/* Check to see if the file is there */
	res = f_stat(filename, NULL);
	if (FR_OK != res)
	{
		tfp_printf("ERROR WTF 1 %d \r\n", res);
#ifdef DEBUG
		tfp_printf("File does not exist. \r\n");
		print_error_code(res);
#endif
		return 1;
	}

#ifdef DEBUG
	tfp_printf("Opening for writing\r\n");
#endif
	/* Open the file for write / append */
	res = f_open(&f, filename, FA_WRITE | FA_OPEN_APPEND);
	if (FR_OK != res)
	{
		tfp_printf("ERROR WTF 2 %d \r\n", res);
#ifdef DEBUG
		tfp_printf("Problem opening file \r\n");
		print_error_code(res);
#endif
		return 1;
	}

	input_phase = FILEDATA;

	/* Wait for FILEDATA phase */
	while (input_phase == FILEDATA){}

	data = (char *)filedata;
	towrite = filedata_char_count-1;
	written = 0;

	while(towrite)
	{
		/* Write data */
		res = f_write(&f, data, towrite, &written);

		if (FR_OK == res)
		{
			f_sync(&f);
			towrite -= written;
			data += written;
#ifdef DEBUG
			tfp_printf("\r\n" "Wrote %ld bytes\r\n", written);
#endif
		}
		else
		{
			tfp_printf("\r\n" "ERROR WTF 3 %d \r\n", res);
#ifdef DEBUG
			tfp_printf("Problem writing data \r\n");
			print_error_code(res);
#endif
			return 1;
		}
	}

	/* write a new line to the file */
	res = f_write(&f, (char *)"\r\n", 2, &written);

	if (FR_OK != res)
	{
		tfp_printf("ERROR WTF 4 %d \r\n", res);
#ifdef DEBUG
		tfp_printf("Problem writing new line \r\n");
		print_error_code(res);
#endif
		return 1;
	}

#ifdef DEBUG
	tfp_printf("Closing \r\n");
#endif
	/* Close the file */
	res = f_close(&f);
	if (FR_OK != res)
	{
		tfp_printf("ERROR WTF 5 %d \r\n", res);
#ifdef DEBUG
		tfp_printf("Error closing \r\n");
		print_error_code(res);
#endif
		return 1;
	}

	reset_vars();
	tfp_printf("\r\n" "> \r\n");
	delayms(100);

	return 0;
}

int8_t cmd_rdf(void){
	/* Command rdf */
	tfp_printf( "\r\n");

	/* Open the file  */
#ifdef DEBUG
	tfp_printf("Opening for reading \r\n");
#endif
	res = f_open(&f, filename, FA_READ);
	if (FR_OK != res)
	{
		tfp_printf("ERROR RDF 1 %d \r\n", res);
#ifdef DEBUG
		tfp_printf("Error opening for reading \r\n");
		print_error_code(res);
#endif
		return 1;
	}

	do
	{
		/* Read the data from the file  */
		f_read(&f, buffer, 128, &read);
		uart_writen(UART0, (uint8_t *)buffer, read);
	}
	while(read == 128);

	/* Close the file  */
#ifdef DEBUG
	tfp_printf("Closing \r\n");
#endif
	res = f_close(&f);
	if (FR_OK != res)
	{
		tfp_printf("ERROR RDF 2 %d \r\n", res);
#ifdef DEBUG
		tfp_printf("Error closing \r\n");
		print_error_code(res);
#endif
		return 1;
	}

	reset_vars();

	tfp_printf("\r\n" "> \r\n");
	delayms(100);
	return 0;
}

int8_t cmd_unm(void){
	/* Command unm  */
	tfp_printf( "\r\n");

	/* Unmount the disk */
#ifdef DEBUG
	tfp_printf("Unmount \r\n\r\n");
#endif
	res = f_mount(0, "", 0);
	if (FR_OK != res)
	{
		tfp_printf("ERROR UNM %d \r\n", res);
#ifdef DEBUG
		tfp_printf("Unable to unmount File System\r\n");
		print_error_code(res);
#endif
		return 1;
	}
	//free(&fs);
	reset_vars();

	delayms(100);
	return 2;
}

int8_t cmd_help(void){
	/* Command ? (help) */
	tfp_printf("\r\n");

	/* List the commands */
	tfp_printf("HELP MENU. \r\n");
	tfp_printf("General Format is: \r\n");
	tfp_printf("<command><space><rtn> \r\n");
	tfp_printf("<command><space><filename/directory name><rtn> \r\n");
	tfp_printf("<command><space><filename/directory name><rtn><data><rtn> \r\n");
	tfp_printf("\r\n");
	tfp_printf("Directory Commands:\r\n");
	tfp_printf("dir <directory><ret>                -List directory contents ('.' is current) \r\n");
	tfp_printf("mkd <directory><ret>                -Make directory \r\n");
	tfp_printf("cd <directory><ret>                 -Change directory \r\n");
	tfp_printf("get <ret>                           -Get current directory (N/A on exFAT) \r\n");
	tfp_printf("\r\n");
	tfp_printf("File Commands:\r\n");
	tfp_printf("crf <filename><ret>                 -Create a new file \r\n");
	tfp_printf("wtf <filename><ret><data><ret>      -Write data to file. Max 1024 chars. \r\n");
	tfp_printf("rdf <filename><ret>                 -Read file contents \r\n");
	tfp_printf("siz <filename><ret>                 -Get file size in bytes \r\n");
	tfp_printf("\r\n");
	tfp_printf("File/Directory Commands:\r\n");
	tfp_printf("del <file/dir><ret>                 -Delete file or EMPTY directory \r\n");
	tfp_printf("rnm <file/dir><rtn><file/dir><rtn>  -Rename/move file or directory \r\n");
	tfp_printf("tim <file/dir><ret> \r\n");
	tfp_printf("<YYYYMMDDHHMMSS><rtn>               -Set timestamp of file or directory \r\n");
	tfp_printf("\r\n");
	tfp_printf("Disk Commands:\r\n");
	tfp_printf("unm <ret>                           -Unmount drive \r\n");
	tfp_printf("gel <ret>                           -Get drive label \r\n");
	tfp_printf("sel <name><ret>                     -Set drive label \r\n");
	tfp_printf("fre <ret>                           -Get free clusters on the volume \r\n");
	tfp_printf("rtc <YYYYMMDDHHMMSS><ret>           -Set the FT90x RTC for timestamp info \r\n");
	tfp_printf("\r\n");
	tfp_printf("Other Commands:\r\n");
	tfp_printf("? <ret>                             -Help menu \r\n");
	tfp_printf("dfu <value><ret>                    -Start the DFU programming interface in ms\r\n");
	tfp_printf("ver <ret>                           -Get the version of this firmware. \r\n");
	tfp_printf("PRESS ESCAPE KEY to abandon any command or filename/directory name entry \r\n");

	reset_vars();
	tfp_printf("\r\n" "> \r\n");
	delayms(100);

	return 0;
}

int8_t cmd_siz(void){
	/* Command siz */
	tfp_printf( "\r\n");

	/* Check to see if the file is there */
	res = f_stat(filename, NULL);
	if (FR_OK != res)
	{
		tfp_printf("ERROR SIZ 1 %d \r\n", res);
#ifdef DEBUG
		tfp_printf("File does not exist. \r\n");
		print_error_code(res);
#endif
		return 1;
	}

	/* Open the file */
#ifdef DEBUG
	tfp_printf("Opening file \r\n");
#endif
	res = f_open(&f, filename, FA_OPEN_EXISTING);
	if (FR_OK != res)
	{
		tfp_printf("ERROR SIZ 2 %d \r\n", res);
#ifdef DEBUG
		tfp_printf("Problem opening file \r\n");
		print_error_code(res);
#endif
		return 1;
	}

	/* Get the size of the file */
	res = f_size(&f);
	if (FR_OK != res)
	{
		tfp_printf("File Size: %d bytes. \r\n", res);
	}

	/* Close the file */
#ifdef DEBUG
	tfp_printf( "Closing \r\n");
#endif
	res = f_close(&f);
	if (FR_OK != res)
	{
		tfp_printf("ERROR SIZ 3 %d \r\n", res);
#ifdef DEBUG
		tfp_printf("Error closing \r\n");
		print_error_code(res);
#endif
		return 1;
	}

	reset_vars();
	tfp_printf("> \r\n");
	delayms(100);

	return 0;
}

int8_t cmd_rnm(void){
	/* Command rnm */
	tfp_printf("\r\n");

	/* Check to see if the file is there */
	res = f_stat(filename, NULL);
	if (FR_OK != res)
	{
		tfp_printf("ERROR RNM 1 %d \r\n", res);
#ifdef DEBUG
		tfp_printf("File does not exist. \r\n");
		print_error_code(res);
#endif
		return 1;
	}

	input_phase = FILEDATA;

	/* Wait for FILEDATA phase */
	while (input_phase == FILEDATA){}

	/* Rename/move the file */
	res = f_rename(filename, filedata);
	if (FR_OK != res)
	{
		tfp_printf("\r\nERROR RNM 2 %d \r\n", res);
#ifdef DEBUG
		tfp_printf("Error renaming \r\n");
		print_error_code(res);
#endif
		reset_vars();
		return 1;
	}

#ifdef DEBUG
	tfp_printf("\r\n" "Rename OK. \r\n");
#endif

	reset_vars();
	tfp_printf("\r\n" "> \r\n");
	delayms(100);

	return 0;
}

int8_t cmd_fre(void){
	/* Command fre */
	tfp_printf("\r\n");

	/* Get sector information */
	res = f_getfree("", &fre_clust, &fs);
	if (FR_OK != res)
	{
		tfp_printf("ERROR FRE %d \r\n", res);
#ifdef DEBUG
		tfp_printf("Get Free space error. \r\n");
		print_error_code(res);
#endif
		return 1;
	}

	/* Get total sectors and free sectors */
	tot_sect = (fs.n_fatent - 2) * fs.csize;
	fre_sect = fre_clust * (fs.csize);

	/* Print the free space (assuming 512 bytes/sector) */
	tfp_printf("%10lu KB total sectors.\r\n%10lu KB free sectors.\r\n",
			tot_sect / 2, fre_sect / 2);

	reset_vars();
	tfp_printf("\r\n" "> \r\n");
	delayms(100);

	return 0;
}

int8_t cmd_gel(void){
	/* Command gel */
	tfp_printf("\r\n");

	/* Get disk label information  */
	res = f_getlabel("", label, 0);
	if (FR_OK != res)
	{
		tfp_printf("ERROR GEL %d \r\n", res);
#ifdef DEBUG
		tfp_printf("Get Label error. \r\n");
		print_error_code(res);
#endif
		return 1;
	}

	/* Print the drive label */
	tfp_printf("Drive label: %s\r\n", label);

	reset_vars();
	tfp_printf("\r\n" "> \r\n");
	delayms(100);

	return 0;
}

int8_t cmd_sel(void){
	/* Command sel */
	tfp_printf("\r\n");

	/* Set disk label information  */
	res = f_setlabel(filename);
	if (FR_OK != res)
	{
		tfp_printf("ERROR SEL %d \r\n", res);
#ifdef DEBUG
		tfp_printf("Set label error. \r\n");
		print_error_code(res);
#endif
		return 1;
	}

#ifdef DEBUG
	tfp_printf("Set Drive Label OK. \r\n");
#endif

	reset_vars();
	tfp_printf("\r\n" "> \r\n");
	delayms(100);

	return 0;
}

int8_t cmd_tim(void){
	/* Command tim  */
	tfp_printf("\r\n");

	int tim_year;
	int tim_month;
	int tim_mday;
	int tim_hour;
	int tim_min;
	int tim_sec;

	/* Check to see if the file is there */
	res = f_stat(filename, NULL);
	if (FR_OK != res)
	{
		tfp_printf("\r\nERROR TIM 1 %d \r\n", res);
#ifdef DEBUG
		tfp_printf("File does not exist. \r\n");
		print_error_code(res);
#endif
		return 1;
	}

	input_phase = FILEDATA;

	/* Wait for FILEDATA phase */
	while (input_phase == FILEDATA){}

	if(filedata_char_count != 15)
	{
		tfp_printf("\r\nERROR TIM 2 \r\n");
#ifdef DEBUG
		tfp_printf("Wrong number of numbers. \r\n");
#endif
		return 1;
	}

	/* Convert characters to numbers and put into relevant variables */
	tim_year = ((filedata[0] - '0')*1000)+((filedata[1] - '0')*100)+((filedata[2] - '0')*10)+((filedata[3] - '0'));
	tim_month = ((filedata[4] - '0')*10)+(filedata[5] - '0');
	tim_mday = ((filedata[6] - '0')*10)+(filedata[7] - '0');
	tim_hour = ((filedata[8] - '0')*10)+(filedata[9] - '0');
	tim_min = ((filedata[10] - '0')*10)+(filedata[11] - '0');
	tim_sec = ((filedata[12] - '0')*10)+(filedata[13] - '0');

	fno.fdate = (WORD)(((tim_year - 1980) * 512U) | tim_month * 32U | tim_mday);
	fno.ftime = (WORD)(tim_hour * 2048U | tim_min * 32U | tim_sec / 2U);

	/* Set the file timestamp data */
	res = f_utime(filename, &fno);
	if (FR_OK != res)
	{
		tfp_printf("\r\nERROR TIM 3 %d \r\n", res);
#ifdef DEBUG
		tfp_printf("Set timestamp error. \r\n");
		print_error_code(res);
#endif
		return 1;
	}

#ifdef DEBUG
	tfp_printf("\r\n" "Set filename/directory time OK. \r\n");
#endif

	reset_vars();
	tfp_printf("\r\n" "> \r\n");
	delayms(100);

	return 0;
}

int8_t cmd_rtc(void){
	/* Command rtc  */
	tfp_printf("\r\n");

	if (HIPID_value == 0x9000001) //RevB
	{
		uint32_t rtcdata;

		struct tm time = { 0 };

		if(filename_char_count != 15)
		{
			tfp_printf("ERROR RTC 1 \r\n");
#ifdef DEBUG
			tfp_printf("Wrong number of numbers. \r\n");
#endif
			return 1;
		}

		rtc_year = ((filename[0] - '0')*1000)+((filename[1] - '0')*100)+((filename[2] - '0')*10)+((filename[3] - '0')) - 1980;
		rtc_month = ((filename[4] - '0')*10)+(filename[5] - '0');
		rtc_mday = ((filename[6] - '0')*10)+(filename[7] - '0');
		rtc_hour = ((filename[8] - '0')*10)+(filename[9] - '0');
		rtc_min = ((filename[10] - '0')*10)+(filename[11] - '0');
		rtc_sec = ((filename[12] - '0')*10)+(filename[13] - '0');

		rtcdata = (((rtc_hour*60*60)+(rtc_min*60)+rtc_sec)<<5);

		time.tm_sec = rtcdata;

		/* List the root directory */
		res = rtc_write(&time);
		if (FR_OK == res)
		{
#ifdef DEBUG
			tfp_printf("RTC Set OK \r\n");
#endif
		}
		else
		{
			tfp_printf("ERROR RTC %d \r\n", res);
#ifdef DEBUG
			tfp_printf("Error Setting RTC \r\n");
#endif
			return 1;
		}

		reset_vars();
		tfp_printf("> \r\n");
		delayms(100);

	}
	else if (HIPID_value == 0x9000002) //RevC
	{
		struct tm time = { 0 };

		if(filename_char_count != 15)
		{
			tfp_printf("\r\n ERROR RTC 1 \r\n");
#ifdef DEBUG
			tfp_printf("\r\n Wrong number of numbers. \r\n");
#endif
			return 1;
		}

		/* Convert characters to numbers and put into relevant variables */
		time.tm_year = ((filename[0] - '0')*1000)+((filename[1] - '0')*100)+((filename[2] - '0')*10)+((filename[3] - '0')) - 2000;
		time.tm_mon = ((filename[4] - '0')*10)+(filename[5] - '0');
		time.tm_mday = ((filename[6] - '0')*10)+(filename[7] - '0');
		time.tm_hour = ((filename[8] - '0')*10)+(filename[9] - '0');
		time.tm_min = ((filename[10] - '0')*10)+(filename[11] - '0');
		time.tm_sec = ((filename[12] - '0')*10)+(filename[13] - '0');

		/* Write to the FT9xx RTC */
		res = rtc_write(&time);
		if (FR_OK == res)
		{
#ifdef DEBUG
			tfp_printf("RTC Set OK \r\n");
#endif
		}
		else
		{
			tfp_printf("ERROR RTC 2 %d\r\n", res);
#ifdef DEBUG
			tfp_printf("Error Setting RTC \r\n");
#endif
			return 1;
		}

		reset_vars();
		tfp_printf("> \r\n");
		delayms(100);
	}

	return 0;
}

int8_t cmd_dfu(void)
{
	/* Command dfu */
	tfp_printf("\r\n");

	int i = filename_char_count-1;
	uint64_t dfu_count = 0;
	uint64_t dfu_multiply = 1;

#ifdef DEBUG
	tfp_printf("DFU Command Received \r\n");
#endif

	/* Process DFU time */
	for(i=filename_char_count-1;i>0;i--)
	{
		dfu_count = dfu_count + ((filename[i-1] - '0')*dfu_multiply);
		dfu_multiply = dfu_multiply*10;
	}

	/* Start USB DFU interface for specified time */
#ifdef DEBUG
	tfp_printf("Interface available for %d ms \r\n", dfu_count);
#endif

	/* Check for a USB device connection and initiate a DFU firmware download or
	   upload operation. This will timeout and return control here if no host PC
	   program contacts the device's DFU interface. USB device mode is disabled
	   before returning.
	 */
	STARTUP_DFU(dfu_count);

#ifdef DEBUG
	tfp_printf("DFU Interface no longer available. Resetting. \r\n");
#endif

#ifdef DEBUG
	tfp_printf("Unmount \r\n\r\n");
#endif
	/* Unmount the drive */
	res = f_mount(0, "", 0);
	if (FR_OK != res)
	{
		tfp_printf("ERROR DFU %d \r\n", res);
#ifdef DEBUG
		tfp_printf("Unable to unmount File System\r\n");
		print_error_code(res);
#endif
		return 1;
	}

	mount_init = 0;

	reset_vars();

	/* Go back to main function to reinitialise everything */
	main();

	return 0;
}

int8_t cmd_ver(void)
{
	/* Command ver */
	tfp_printf("\r\n");

	tfp_printf("Version 1.0 \r\n");

	reset_vars();

	tfp_printf("\r\n" "> \r\n");
	delayms(100);

	return 0;
}

void print_error_code(int result)
{
	/* Print error codes */
	switch(result)
	{
	case 1:
		tfp_printf("Error Code 1: FR_DISK_ERR \r\n");
		break;
	case 2:
		tfp_printf("Error Code 2: FR_INT_ERR \r\n");
		break;
	case 3:
		tfp_printf("Error Code 3: FR_NOT_READY \r\n");
		break;
	case 4:
		tfp_printf("Error Code 4: FR_NO_FILE \r\n");
		break;
	case 5:
		tfp_printf("Error Code 5: FR_NO_PATH \r\n");
		break;
	case 6:
		tfp_printf("Error Code 6: FR_INVALID_NAME \r\n");
		break;
	case 7:
		tfp_printf("Error Code 7: FR_DENIED \r\n");
		break;
	case 8:
		tfp_printf("Error Code 8: FR_EXIST \r\n");
		break;
	case 9:
		tfp_printf("Error Code 9: FR_INVALID_OBJECT \r\n");
		break;
	case 10:
		tfp_printf("Error Code 10: FR_WRITE_PROTECTED \r\n");
		break;
	case 11:
		tfp_printf("Error Code 11: FR_INVALID_DRIVE \r\n");
		break;
	case 12:
		tfp_printf("Error Code 12: FR_NOT_ENABLED \r\n");
		break;
	case 13:
		tfp_printf("Error Code 13: FR_NO_FILESYSTEM \r\n");
		break;
	case 14:
		tfp_printf("Error Code 14: FR_MKFS_ABORTED \r\n");
		break;
	case 15:
		tfp_printf("Error Code 15: FR_TIMEOUT \r\n");
		break;
	case 16:
		tfp_printf("Error Code 16: FR_LOCKED \r\n");
		break;
	case 17:
		tfp_printf("Error Code 17: FR_NOT_ENOUGH_CORE \r\n");
		break;
	case 18:
		tfp_printf("Error Code 18: FR_TOO_MANY_OPEN_FILES \r\n");
		break;
	case 19:
		tfp_printf("Error Code 19: FR_INVALID_PARAMETER \r\n");
		break;
	default:
		tfp_printf("Unknown Error \r\n");
		break;
	}
}

void hub_scan_for_boms(USBH_device_handle hDev, int level)
{
	uint8_t status;
	USBH_device_handle hNext;
	USBH_interface_handle hInterface;
	uint8_t usbClass;
	uint8_t usbSubclass;
	uint8_t usbProtocol;

	do {
		/* Search all the interfaces on this device */
		status = USBH_get_interface_list(hDev, &hInterface);
		while (status == USBH_OK)
		{
			if (USBH_interface_get_class_info(hInterface, &usbClass, &usbSubclass, &usbProtocol) == USBH_OK)
			{
				/* If it is a MASS STORAGE interface then use that */
				if ((usbClass == USB_CLASS_MASS_STORAGE) &&
						(usbSubclass == USB_SUBCLASS_MASS_STORAGE_SCSI) &&
						(usbProtocol == USB_PROTOCOL_MASS_STORAGE_BOMS))
				{
					/* Set LEDs */
					gpio_write(4, 1);
					gpio_write(5, 0);
#ifdef DEBUG
					tfp_printf("BOMS device found at level %d\r\n", level);
#endif
					tfp_printf("\r\n" "> \r\n");
					while ((test_res == 0 || test_res == 1)&&(USBH_interface_get_class_info(hInterface, &usbClass, &usbSubclass, &usbProtocol) == USBH_OK))
					{
						test_res = fs_testing(hInterface);
						if (test_res == 1)
						{
							reset_vars();
							tfp_printf("\r\n" "> \r\n");
						}
					}

					mount_init = 0;

					reset_vars();
				}
			}
			/* Next interface on this device */
			status = USBH_get_next_interface(hInterface, &hInterface);
		}

		/* Get any child devices of this device (i.e. this is a hub). */
		status = USBH_get_device_list(hDev, &hNext);
		if (status == USBH_OK)
		{
			/* Recursively scan */
			hub_scan_for_boms(hNext, level + 1);
		}

		/* Get any other devices on the same device. */
		status = USBH_get_next_device(hDev, &hNext);
		if (status == USBH_OK)
		{
			hDev = hNext;
		}

	} while (status == USBH_OK);
}

uint8_t usbh_testing(void)
{
	uint8_t status;
	USBH_device_handle hRootDev;
	USBH_STATE connect;

	USBH_initialise(NULL);

	while (1)
	{
		USBH_get_connect_state(USBH_ROOT_HUB_HANDLE, USBH_ROOT_HUB_PORT, &connect);
		if (connect == USBH_STATE_NOTCONNECTED)
		{
#ifdef DEBUG
			tfp_printf("\r\nPlease plug in a USB Device\r\n");
#endif

			/* Detect usb device connect */
			do
			{
				/* Toggle LEDs */
				toggle_LEDs();

				status = USBH_process();
				USBH_get_connect_state(USBH_ROOT_HUB_HANDLE, USBH_ROOT_HUB_PORT, &connect);
			} while (connect == USBH_STATE_NOTCONNECTED);
		}
#ifdef DEBUG
		tfp_printf("\r\nUSB Device Detected\r\n");
#endif

		do{
			status = USBH_process();
			USBH_get_connect_state(USBH_ROOT_HUB_HANDLE, USBH_ROOT_HUB_PORT, &connect);
		} while (connect != USBH_STATE_ENUMERATED);
#ifdef DEBUG
		tfp_printf("USB Device Enumerated\r\n");
#endif

		/* Get the first device (device on root hub) */
		status = USBH_get_device_list(USBH_ROOT_HUB_HANDLE, &hRootDev);
		if (status != USBH_OK)
		{
			/* Report the error code. */
			tfp_printf("ERROR CLOSE \r\n");
#ifdef DEBUG
			tfp_printf("%d\r\n", status);
#endif
		}
		else
		{
			/* Perform the actual application. */
			hub_scan_for_boms(hRootDev, 1);
		}

		/* Set LEDs */
		gpio_write(4, 0);
		gpio_write(5, 1);

#ifdef DEBUG
		tfp_printf("\r\nPlease remove the USB Device\r\n");
#endif
		/* Detect usb device disconnect */
		do
		{
			status = USBH_process();
			USBH_get_connect_state(USBH_ROOT_HUB_HANDLE, USBH_ROOT_HUB_PORT, &connect);
		} while (connect != USBH_STATE_NOTCONNECTED);
	}
	return 0;
}

void process_uart_data()
{

	switch(input_phase) {
	case COMMAND:
		/* COMMAND phase */
		/* Read the UART receive data */
		uartrb_getc(UART0, &command[command_char_count]);

		/* If <ret> or <space> received, move to the FILENAME phase */
		if (command[command_char_count] == '\r' || command[command_char_count] == ' ')
		{
			input_phase = FILENAME;
		}
		command_char_count++;

		/* If <escape> received, abort and reset variables */
		if (command[command_char_count-1] == 0x1b)
		{
			tfp_printf("\r\n" "EESC \r\n");
#ifdef DEBUG
			tfp_printf("ESCAPE pressed. Resetting. \r\n");
#endif
			tfp_printf("> \r\n");
			reset_vars();
		}
		break;
	case FILENAME:
		/* FILENAME phase */
		/* Read the UART receive data */
		uartrb_getc(UART0, &filename[filename_char_count]);

		/* If <ret> received, add a NULL terminate */
		if (filename[filename_char_count] == '\r')
		{
			filename[filename_char_count+1] = '\0';
			command_entered = TRUE;
		}
		filename_char_count++;

		/* If <escape> received, abort and reset variables */
		if (filename[filename_char_count-1] == 0x1b)
		{
			tfp_printf("\r\n" "EESC \r\n");
#ifdef DEBUG
			tfp_printf("ESCAPE pressed. Resetting. \r\n");
#endif
			tfp_printf("> \r\n");
			reset_vars();
		}
		break;
	case FILEDATA:
		/* FILEDATA phase */
		/* Read the UART receive data */
		uartrb_getc(UART0, &filedata[filedata_char_count]);

		/* If <ret> received or maximum characters, move to CONT_FILEDATA phase */
		if ((filedata[filedata_char_count] == '\r')||(filedata_char_count>=1024))
		{
			input_phase = CONT_FILEDATA;
		}
		filedata_char_count++;

		/* If <escape> received, abort and reset variables */
		if (filedata[filedata_char_count-1] == 0x1b)
		{
			tfp_printf("\r\n" "EESC \r\n");
#ifdef DEBUG
			tfp_printf("ESCAPE pressed. Resetting. \r\n");
#endif
			tfp_printf("> \r\n");
			reset_vars();
		}
		break;
	case CONT_FILEDATA:
		/* CONT_FILEDATA phase */
		//do nothing
		break;
	default:
		tfp_printf("ERROR \r\n");
#ifdef DEBUG
		tfp_printf("ERROR. Resetting. \r\n");
#endif
		tfp_printf("> \r\n");
		reset_vars();
		break;
	}
}

void toggle_LEDs(void)
{
	static uint8_t state;

	/* Toggle LEDs on the Ethernet Ports as these are the only ones available on the MM900EVxB */
	gpio_write(4, state);
	gpio_write(5, !state);

	state = !state;

	delayms(100);
}

void enable_interrupt()
{

	/* Attach the interrupt so it can be called... */
	interrupt_attach(interrupt_timers, (int8_t)interrupt_timers, ISR_timer);
	/* Enable interrupts to be fired... */
	uart_enable_interrupts_globally(UART0);

	/* Enable interrupt handling. */
	interrupt_enable_globally();
}

/* FUNCTIONS ***********************************************************************/

int main(int argc, char *argv[])
{

	/* Enable the UART Device... */
	sys_enable(sys_device_uart0);

	/* Make GPIO48 function as UART0_TXD and GPIO49 function as UART0_RXD... */
	gpio_function(48, pad_uart0_txd); /* UART0 TXD */
	gpio_function(49, pad_uart0_rxd); /* UART0 RXD */
	gpio_function(50, pad_uart0_rts); /* UART0 RTS */
	gpio_function(51, pad_uart0_cts); /* UART0 CTS */
	uart_open(UART0,                    /* Device */
			1,                        /* Prescaler = 1 */
			UART_DIVIDER_19200_BAUD,  /* Divider = 1302 */
			uart_data_bits_8,         /* No. Data Bits */
			uart_parity_none,         /* Parity */
			uart_stop_bits_1);        /* No. Stop Bits */

	uartrb_setup(UART0, uartrb_flow_none); //flow control handled in uartrb code

	/* Ethernet LEDs used for indication as these are the only ones available on the MM900EVxB */
	gpio_function(4, pad_gpio4); /* GPIO4 Pad function 0 */
	gpio_dir(4, pad_dir_output);
	gpio_pull(4, pad_pull_none);
	gpio_function(5, pad_gpio5); /* GPIO5 Pad function 0 */
	gpio_dir(5, pad_dir_output);
	gpio_pull(5, pad_pull_none);

	//Get Silicon ID
	HIPID_value = SYS->HIPID;

#ifdef RTC_ENABLE
	/* Enable the RTC... */
	sys_enable(sys_device_timer_wdt);

	/* Initialise the RTC to run indefinitely... */
	rtc_init();

	/* Start the RTC... */
	rtc_start();
#endif

#ifdef DEBUG
	/* Print out a welcome message... */
	uart_puts(UART0,
			"\x1B[2J" /* ANSI/VT100 - Clear the Screen */
			"\x1B[H"  /* ANSI/VT100 - Move Cursor to Home */
	);
#endif

	/* Enable tfp_printf() functionality... */
	init_printf(UART0, tfp_putc);

	sys_enable(sys_device_timer_wdt);

	enable_interrupt();


	/* Timer A = 1ms */
	timer_prescaler(1000);
	timer_init(timer_select_a, 1000, timer_direction_down, timer_prescaler_select_on, timer_mode_continuous);
	timer_enable_interrupt(timer_select_a);
	timer_start(timer_select_a);

	/* Timer B for RTC additional functionality */
	timer_init(timer_select_b, 0xFFFF, timer_direction_down, timer_prescaler_select_on, timer_mode_continuous);
	timer_enable_interrupt(timer_select_b);
	timer_start(timer_select_b);

#ifdef DEBUG
	tfp_printf("Copyright (C) Bridgetek Pte Ltd \r\n"
			"--------------------------------------------------------------------- \r\n"
			"Welcome to the UART to BOMs example... \r\n"
			"\r\n"
			"Find Flash Disk devices connected to the USB \r\n"
			"Host Port and allow control via UART0. \r\n"
			"--------------------------------------------------------------------- \r\n"
	);

#endif

	usbh_testing();

	interrupt_detach(interrupt_timers);
	interrupt_detach(interrupt_uart0);
	interrupt_disable_globally();

	sys_disable(sys_device_timer_wdt);

	/* Wait froever */
	while(1);

	return 0;
}
